
#GPUCALC_GNU_COMPILED = 1

Enumeration
  #GPUCALC_ERR_NO_ERROR
  #GPUCALC_ERR_RENDERINGCONTEXT
  #GPUCALC_ERR_COMPILE_VERTEX
  #GPUCALC_ERR_COMPILE_FRAGMENT
  #GPUCALC_ERR_LINK_SHADERS
  #GPUCALC_ERR_MULTITEXTURING
  #GPUCALC_ERR_SHADERS
  #GPUCALC_ERR_FRAMEBUFFEROBJECTS
  #GPUCALC_ERR_VERTEXBUFFEROBJECTS
  #GPUCALC_ERR_FLOATTEXTURE
  #GPUCALC_ERR_INTEGERTEXTURE
  #GPUCALC_ERR_INVALIDDATATYPE
EndEnumeration

Enumeration
  #GPUCALC_TYPE_FLOAT32
  #GPUCALC_TYPE_UINT32
  #GPUCALC_TYPE_UINT16
  #GPUCALC_TYPE_UINT8
  #GPUCALC_TYPE_INT32
  #GPUCALC_TYPE_INT16
  #GPUCALC_TYPE_INT8
EndEnumeration

Structure DATASETMEMORY
  StructureUnion
    float.f[0]
    
    uint.l[0]
    ushort.u[0]
    ubyte.a[0]
    
    int.l[0]
    short.w[0]
    byte.b[0]
  EndStructureUnion
EndStructure

Interface GPUCalcContext
  loadCode(index.i, code.p-ascii, length.i)
  loadDataset(index.i, dataType.i, *dataset.DATASETMEMORY, elementCount.i)
  bindCode(index.i)
  unbindCode()
  calculate(*result.DATASETMEMORY, elementCount.i)
  hasFinishedLastCommand.b()
  wasLastCommandSuccessful.b()
  getLastErrorCode.i()
  getCompilerErrorVertex.i()
  getCompilerErrorFragment.i()
  getLinkerError.i()
  allocateDatasetMemory.i()
  getGLSLVersion.i()
  integerCalculationSupported.b()
EndInterface

Prototype.i _gpuCalcCreateContext(type.i, maxDatasetCount.i, maxCodeCount.i)
Prototype   _gpuCalcReleaseContext(*context.GPUCalcContext)
Prototype   _gpuCalcReleaseDataset(*dataset.DATASETMEMORY)
Prototype   _gpuCalcYield()

Global gpuCalcCreateContext._gpuCalcCreateContext
Global gpuCalcReleaseContext._gpuCalcReleaseContext
Global gpuCalcReleaseDataset._gpuCalcReleaseDataset
Global gpuCalcYield._gpuCalcYield

Procedure LoadGPUCalcSymbols()
  Shared gpuCalcLibraryHandle

  If gpuCalcLibraryHandle = 0
    CompilerIf #PB_Compiler_OS = #PB_OS_Windows
      CompilerIf #PB_Compiler_Processor = #PB_Processor_x86
        gpuCalcLibraryHandle = OpenLibrary(#PB_Any, "GPUCalc32.dll")
      CompilerElse
        gpuCalcLibraryHandle = OpenLibrary(#PB_Any, "GPUCalc64.dll")
      CompilerEndIf
    CompilerElse
      CompilerIf #PB_Compiler_Processor = #PB_Processor_x86
        gpuCalcLibraryHandle = OpenLibrary(#PB_Any, GetCurrentDirectory() + "GPUCalc32.so")
      CompilerElse
        gpuCalcLibraryHandle = OpenLibrary(#PB_Any, GetCurrentDirectory() + "GPUCalc64.so")
      CompilerEndIf
    CompilerEndIf
  EndIf

  If gpuCalcLibraryHandle
    CompilerIf #PB_Compiler_OS = #PB_OS_Windows
      CompilerIf #GPUCALC_GNU_COMPILED
        CompilerIf #PB_Compiler_Processor = #PB_Processor_x86
        gpuCalcCreateContext  = GetFunction(gpuCalcLibraryHandle, "gpuCalcCreateContext@12")
        gpuCalcReleaseContext = GetFunction(gpuCalcLibraryHandle, "gpuCalcReleaseContext@4")
        gpuCalcReleaseDataset = GetFunction(gpuCalcLibraryHandle, "gpuCalcReleaseDataset@4")
        gpuCalcYield          = GetFunction(gpuCalcLibraryHandle, "gpuCalcYield@0")
        CompilerElse
        gpuCalcCreateContext  = GetFunction(gpuCalcLibraryHandle, "gpuCalcCreateContext")
        gpuCalcReleaseContext = GetFunction(gpuCalcLibraryHandle, "gpuCalcReleaseContext")
        gpuCalcReleaseDataset = GetFunction(gpuCalcLibraryHandle, "gpuCalcReleaseDataset")
        gpuCalcYield          = GetFunction(gpuCalcLibraryHandle, "gpuCalcYield")
        CompilerEndIf
        CompilerElse
        gpuCalcCreateContext  = GetFunction(gpuCalcLibraryHandle, "_gpuCalcCreateContext@12")
        gpuCalcReleaseContext = GetFunction(gpuCalcLibraryHandle, "_gpuCalcReleaseContext@4")
        gpuCalcReleaseDataset = GetFunction(gpuCalcLibraryHandle, "_gpuCalcReleaseDataset@4")
        gpuCalcYield          = GetFunction(gpuCalcLibraryHandle, "_gpuCalcYield@0")
      CompilerEndIf
    CompilerElse
      CompilerIf #GPUCALC_GNU_COMPILED
        gpuCalcCreateContext  = GetFunction(gpuCalcLibraryHandle, "gpuCalcCreateContext")
        gpuCalcReleaseContext = GetFunction(gpuCalcLibraryHandle, "gpuCalcReleaseContext")
        gpuCalcReleaseDataset = GetFunction(gpuCalcLibraryHandle, "gpuCalcReleaseDataset")
        gpuCalcYield          = GetFunction(gpuCalcLibraryHandle, "gpuCalcYield")
      CompilerEndIf
    CompilerEndIf
  EndIf
  
  If Not (gpuCalcCreateContext <> 0 And gpuCalcReleaseContext <> 0 And gpuCalcReleaseDataset <> 0 And gpuCalcYield <> 0)
    If gpuCalcLibraryHandle
      CloseLibrary(gpuCalcLibraryHandle)
    EndIf
    ProcedureReturn 0
  EndIf

  ProcedureReturn 1
EndProcedure

Procedure ReleaseGPUCalcSymbols()
  Shared gpuCalcLibraryHandle

  If gpuCalcLibraryHandle <> 0
    CloseLibrary(gpuCalcLibraryHandle)
  EndIf

  gpuCalcCreateContext  = 0
  gpuCalcReleaseContext = 0
  gpuCalcReleaseDataset = 0
  gpuCalcYield          = 0
EndProcedure
; IDE Options = PureBasic 4.60 Beta 3 (Windows - x64)
; CursorPosition = 56
; FirstLine = 19
; Folding = -
; EnableXP